unit MMTimer;

{
property EventKind: TTimerEventKind;
ekOneShot̎Interval~bɂP񂾂Cxg܂B
ekPeriodic̎Interval~bƂɃCxg܂B

property Resolution: Cardinal;
^C}Cxg̕\w肵܂B
Interval͂̒lȏłȂĂ͂܂B
̒lقǃVXeׂ܂܂B

property MinResolution: Cardinal; (ReadOnly)
Resolution̎邱Ƃ̂łŏlB
@ˑB

property MaxResolution: Cardinal; (ReadOnly)
Resolution̎邱Ƃ̂łőlB
@ˑB

TTimerƓlB
}


interface

uses
  Windows, Classes, Consts, MMSystem;

type
  TTimerEventKind = (ekOneShot, ekPeriodic);

  TMMTimer = class(TComponent)
  private
    FEnabled: Boolean;
    FEventKind: TTimerEventKind;
    FIDEvent: Cardinal;
    FInterval: Cardinal;
    FMinResolution: Cardinal;
    FMaxResolution: Cardinal;
    FOnTimer: TNotifyEvent;
    FPrevResolution: Cardinal;
    FResolution: Cardinal;
    procedure SetEnabled(Value: Boolean);
    procedure SetEventKind(const Value: TTimerEventKind);
    procedure SetInterval(Value: Cardinal);
    procedure SetOnTimer(Value: TNotifyEvent);
    procedure SetResolution(const Value: Cardinal);
    procedure UpdateTimer;
  protected
    procedure Timer; dynamic;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property EventKind: TTimerEventKind read FEventKind write SetEventKind default ekPeriodic;
    property Interval: Cardinal read FInterval write SetInterval;
    property OnTimer: TNotifyEvent read FOnTimer write SetOnTimer;
    property Resolution: Cardinal read FResolution write SetResolution;
    property MinResolution: Cardinal read FMinResolution;
    property MaxResolution: Cardinal read FMaxResolution;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('External', [TMMTimer]);
end;

procedure TimeProc(IDEvent, uReserved, dwUser, dwReserved1, dwReserved2: Cardinal); stdcall;
begin
  if Assigned(TMMTimer(dwUser).OnTimer) then
    TMMTimer(dwUser).OnTimer(TMMTimer(dwUser));

  if TMMTimer(dwUser).FEventKind = ekOneShot then
    TMMTimer(dwUser).Enabled := False;
end;

{ TMMTimer }

constructor TMMTimer.Create(AOwner: TComponent);
var
  TC: TTimeCaps;
begin
  inherited Create(AOwner);
  FEnabled := True;
  FIDEvent := 0;
  FOntimer := nil;
  FEventKind := ekPeriodic;

  timeGetDevCaps(@TC, SizeOf(TIMECAPS));

  FInterval := TC.wPeriodMax;
  FMaxResolution := TC.wPeriodMax;

  FResolution := TC.wPeriodMin;
  FMinResolution := TC.wPeriodMin;

  FPrevResolution := FResolution;

  UpdateTimer;
end;

destructor TMMTimer.Destroy;
begin
  FEnabled := False;
  UpdateTimer;

  inherited Destroy;
end;

procedure TMMTimer.SetEnabled(Value: Boolean);
begin
  if Value <> FEnabled then
  begin
    FEnabled := Value;
    UpdateTimer;
  end;
end;

procedure TMMTimer.SetEventKind(const Value: TTimerEventKind);
begin
  if FEventKind <> Value then
  begin
    FEventKind := Value;
    UpdateTimer;
  end;
end;

procedure TMMTimer.SetInterval(Value: Cardinal);
begin
  if (Value <> FInterval) and (Value >= FResolution) then
  begin
    FInterval := Value;
    UpdateTimer;
  end;
end;

procedure TMMTimer.SetOnTimer(Value: TNotifyEvent);
begin
  FOnTimer := Value;
  UpdateTimer;
end;

procedure TMMTimer.SetResolution(const Value: Cardinal);
begin
  if (Value <> FResolution) and (Value >= FMinResolution) then
  begin
    FResolution := Value;
    UpdateTimer;
  end;
end;

procedure TMMTimer.Timer;
begin
  if Assigned(FOnTimer) then FOnTimer(Self);
end;

procedure TMMTimer.UpdateTimer;
const
  CallbackType: array[TTimerEventKind] of Cardinal = (TIME_ONESHOT, TIME_PERIODIC);
begin
  if FIDEvent <> 0 then
  begin
    timeKillEvent(FIDEvent);
    timeEndPeriod(FPrevResolution);
    FIDEvent := 0;
  end;

  if (FResolution <> 0) and FEnabled and Assigned(FOnTimer) then
  begin
    timeBeginPeriod(FResolution);
    FIDEvent := timeSetEvent(FInterval, FResolution, TimeProc, DWORD(Self), CallbackType[EventKind]);
    if FIDEvent = 0 then raise EOutOfResources.Create(SNoTimers)
    else FPrevResolution := FResolution;
  end;
end;

end.
